@@ -23,7 +23,7 @@ BEGIN {
 
 BEGIN { eval q{ use vars qw($VERSION $_warning) } }
 
-$VERSION = sprintf '%d.%02d', q$Revision: 0.58 $ =~ m/(\d+)/xmsg;
+$VERSION = sprintf '%d.%02d', q$Revision: 0.60 $ =~ m/(\d+)/xmsg;
 
 # poor Symbol.pm - substitute of real Symbol.pm
 BEGIN {
@@ -167,60 +167,6 @@ if ($^O =~ /\A (?: MSWin32 | NetWare | symbian | dos ) \z/oxms) {
     }
 }
 
-#
-# prepare Latin-1 characters per length
-#
-my @chars1 = ();
-my @chars2 = ();
-my @chars3 = ();
-my @chars4 = ();
-if (exists $range_tr{1}) {
-    my @ranges = @{ $range_tr{1} };
-    while (my @range = splice(@ranges,0,1)) {
-        for my $oct0 (@{$range[0]}) {
-            push @chars1, pack 'C', $oct0;
-        }
-    }
-}
-if (exists $range_tr{2}) {
-    my @ranges = @{ $range_tr{2} };
-    while (my @range = splice(@ranges,0,2)) {
-        for my $oct0 (@{$range[0]}) {
-            for my $oct1 (@{$range[1]}) {
-                push @chars2, pack 'CC', $oct0,$oct1;
-            }
-        }
-    }
-}
-if (exists $range_tr{3}) {
-    my @ranges = @{ $range_tr{3} };
-    while (my @range = splice(@ranges,0,3)) {
-        for my $oct0 (@{$range[0]}) {
-            for my $oct1 (@{$range[1]}) {
-                for my $oct2 (@{$range[2]}) {
-                    push @chars3, pack 'CCC', $oct0,$oct1,$oct2;
-                }
-            }
-        }
-    }
-}
-if (exists $range_tr{4}) {
-    my @ranges = @{ $range_tr{4} };
-    while (my @range = splice(@ranges,0,4)) {
-        for my $oct0 (@{$range[0]}) {
-            for my $oct1 (@{$range[1]}) {
-                for my $oct2 (@{$range[2]}) {
-                    for my $oct3 (@{$range[3]}) {
-                        push @chars4, pack 'CCCC', $oct0,$oct1,$oct2,$oct3;
-                    }
-                }
-            }
-        }
-    }
-}
-my @minchar = (undef, $chars1[ 0], $chars2[ 0], $chars3[ 0], $chars4[ 0]);
-my @maxchar = (undef, $chars1[-1], $chars2[-1], $chars3[-1], $chars4[-1]);
-
 #
 # Latin-1 split
 #
@@ -571,7 +517,12 @@ sub Elatin1::rindex($$;$) {
     sub Elatin1::lc(@) {
         if (@_) {
             my $s = shift @_;
-            return join('', map {defined($lc{$_}) ? $lc{$_} : $_} ($s =~ m/\G ($q_char) /oxmsg)), @_;
+            if (@_ and wantarray) {
+                return join('', map {defined($lc{$_}) ? $lc{$_} : $_} ($s =~ m/\G ($q_char) /oxmsg)), @_;
+            }
+            else {
+                return join('', map {defined($lc{$_}) ? $lc{$_} : $_} ($s =~ m/\G ($q_char) /oxmsg));
+            }
         }
         else {
             return Elatin1::lc_();
@@ -634,7 +585,12 @@ sub Elatin1::rindex($$;$) {
     sub Elatin1::uc(@) {
         if (@_) {
             my $s = shift @_;
-            return join('', map {defined($uc{$_}) ? $uc{$_} : $_} ($s =~ m/\G ($q_char) /oxmsg)), @_;
+            if (@_ and wantarray) {
+                return join('', map {defined($uc{$_}) ? $uc{$_} : $_} ($s =~ m/\G ($q_char) /oxmsg)), @_;
+            }
+            else {
+                return join('', map {defined($uc{$_}) ? $uc{$_} : $_} ($s =~ m/\G ($q_char) /oxmsg));
+            }
         }
         else {
             return Elatin1::uc_();
@@ -652,9 +608,145 @@ sub Elatin1::rindex($$;$) {
 # Latin-1 regexp capture
 #
 {
+###if MULTIBYTE_ANCHORING
+    # 10.3. Creating Persistent Private Variables
+    # in Chapter 10. Subroutines
+    # of ISBN 0-596-00313-7 Perl Cookbook, 2nd Edition.
+
+    my $last_s_matched = 0;
+
+###endif
     sub Elatin1::capture($) {
+###if MULTIBYTE_ANCHORING
+        if ($last_s_matched and ($_[0] =~ m/\A [1-9][0-9]* \z/oxms)) {
+            return $_[0] + 1;
+        }
+###endif
         return $_[0];
     }
+###if MULTIBYTE_ANCHORING
+
+    # Latin-1 regexp mark last m// or qr// matched
+    sub Elatin1::m_matched() {
+        $last_s_matched = 0;
+    }
+
+    # Latin-1 regexp mark last s/// or qr matched
+    sub Elatin1::s_matched() {
+        $last_s_matched = 1;
+    }
+
+    # which matched of m// or s/// at last
+
+    # P.854 31.17. use re
+    # in Chapter 31. Pragmatic Modules
+    # of ISBN 0-596-00027-8 Programming Perl Third Edition.
+
+    @Elatin1::m_matched = (qr/(?{Elatin1::m_matched})/);
+    @Elatin1::s_matched = (qr/(?{Elatin1::s_matched})/);
+###endif
+}
+
+#
+# prepare Latin-1 characters per length
+#
+
+# 1 octet characters
+my @chars1 = ();
+sub chars1 {
+    if (@chars1) {
+        return @chars1;
+    }
+    if (exists $range_tr{1}) {
+        my @ranges = @{ $range_tr{1} };
+        while (my @range = splice(@ranges,0,1)) {
+            for my $oct0 (@{$range[0]}) {
+                push @chars1, pack 'C', $oct0;
+            }
+        }
+    }
+    return @chars1;
+}
+
+# 2 octets characters
+my @chars2 = ();
+sub chars2 {
+    if (@chars2) {
+        return @chars2;
+    }
+    if (exists $range_tr{2}) {
+        my @ranges = @{ $range_tr{2} };
+        while (my @range = splice(@ranges,0,2)) {
+            for my $oct0 (@{$range[0]}) {
+                for my $oct1 (@{$range[1]}) {
+                    push @chars2, pack 'CC', $oct0,$oct1;
+                }
+            }
+        }
+    }
+    return @chars2;
+}
+
+# 3 octets characters
+my @chars3 = ();
+sub chars3 {
+    if (@chars3) {
+        return @chars3;
+    }
+    if (exists $range_tr{3}) {
+        my @ranges = @{ $range_tr{3} };
+        while (my @range = splice(@ranges,0,3)) {
+            for my $oct0 (@{$range[0]}) {
+                for my $oct1 (@{$range[1]}) {
+                    for my $oct2 (@{$range[2]}) {
+                        push @chars3, pack 'CCC', $oct0,$oct1,$oct2;
+                    }
+                }
+            }
+        }
+    }
+    return @chars3;
+}
+
+# 4 octets characters
+my @chars4 = ();
+sub chars4 {
+    if (@chars4) {
+        return @chars4;
+    }
+    if (exists $range_tr{4}) {
+        my @ranges = @{ $range_tr{4} };
+        while (my @range = splice(@ranges,0,4)) {
+            for my $oct0 (@{$range[0]}) {
+                for my $oct1 (@{$range[1]}) {
+                    for my $oct2 (@{$range[2]}) {
+                        for my $oct3 (@{$range[3]}) {
+                            push @chars4, pack 'CCCC', $oct0,$oct1,$oct2,$oct3;
+                        }
+                    }
+                }
+            }
+        }
+    }
+    return @chars4;
+}
+
+# minimum value of each octet
+my @minchar = ();
+sub minchar {
+    if (defined $minchar[$_[0]]) {
+        return $minchar[$_[0]];
+    }
+    $minchar[$_[0]] = (&{(sub {}, \&chars1, \&chars2, \&chars3, \&chars4)[$_[0]]})[0];
+}
+
+# maximum value of each octet
+my @maxchar = ();
+sub maxchar {
+    if (defined $maxchar[$_[0]]) {
+        return $maxchar[$_[0]];
+    }
+    $maxchar[$_[0]] = (&{(sub {}, \&chars1, \&chars2, \&chars3, \&chars4)[$_[0]]})[-1];
 }
 
 #
@@ -727,50 +819,50 @@ sub _charlist_tr {
         # range of multiple octet code
         if (length($char[$i-1]) == 1) {
             if (length($char[$i+1]) == 1) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} @chars1;
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars1();
             }
             elsif (length($char[$i+1]) == 2) {
-                push @range, grep {$char[$i-1] le $_}                           @chars1;
-                push @range, grep {$_ le $char[$i+1]}                           @chars2;
+                push @range, grep {$char[$i-1] le $_}                           &chars1();
+                push @range, grep {$_ le $char[$i+1]}                           &chars2();
             }
             elsif (length($char[$i+1]) == 3) {
-                push @range, grep {$char[$i-1] le $_}                           @chars1;
-                push @range,                                                    @chars2;
-                push @range, grep {$_ le $char[$i+1]}                           @chars3;
+                push @range, grep {$char[$i-1] le $_}                           &chars1();
+                push @range,                                                    &chars2();
+                push @range, grep {$_ le $char[$i+1]}                           &chars3();
             }
             elsif (length($char[$i+1]) == 4) {
-                push @range, grep {$char[$i-1] le $_}                           @chars1;
-                push @range,                                                    @chars2;
-                push @range,                                                    @chars3;
-                push @range, grep {$_ le $char[$i+1]}                           @chars4;
+                push @range, grep {$char[$i-1] le $_}                           &chars1();
+                push @range,                                                    &chars2();
+                push @range,                                                    &chars3();
+                push @range, grep {$_ le $char[$i+1]}                           &chars4();
             }
         }
         elsif (length($char[$i-1]) == 2) {
             if (length($char[$i+1]) == 2) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} @chars2;
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars2();
             }
             elsif (length($char[$i+1]) == 3) {
-                push @range, grep {$char[$i-1] le $_}                           @chars2;
-                push @range, grep {$_ le $char[$i+1]}                           @chars3;
+                push @range, grep {$char[$i-1] le $_}                           &chars2();
+                push @range, grep {$_ le $char[$i+1]}                           &chars3();
             }
             elsif (length($char[$i+1]) == 4) {
-                push @range, grep {$char[$i-1] le $_}                           @chars2;
-                push @range,                                                    @chars3;
-                push @range, grep {$_ le $char[$i+1]}                           @chars4;
+                push @range, grep {$char[$i-1] le $_}                           &chars2();
+                push @range,                                                    &chars3();
+                push @range, grep {$_ le $char[$i+1]}                           &chars4();
             }
         }
         elsif (length($char[$i-1]) == 3) {
             if (length($char[$i+1]) == 3) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} @chars3;
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars3();
             }
             elsif (length($char[$i+1]) == 4) {
-                push @range, grep {$char[$i-1] le $_}                           @chars3;
-                push @range, grep {$_ le $char[$i+1]}                           @chars4;
+                push @range, grep {$char[$i-1] le $_}                           &chars3();
+                push @range, grep {$_ le $char[$i+1]}                           &chars4();
             }
         }
         elsif (length($char[$i-1]) == 4) {
             if (length($char[$i+1]) == 4) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} @chars4;
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars4();
             }
         }
 
@@ -917,41 +1009,41 @@ sub _charlist {
             elsif (length($char[$i-1]) == 1) {
                 if (length($char[$i+1]) == 2) {
                     push @charlist,
-                        _octets(1, $char[$i-1], $maxchar[1], $modifier),
-                        _octets(2, $minchar[2], $char[$i+1], $modifier);
+                        _octets(1, $char[$i-1], &maxchar(1), $modifier),
+                        _octets(2, &minchar(2), $char[$i+1], $modifier);
                 }
                 elsif (length($char[$i+1]) == 3) {
                     push @charlist,
-                        _octets(1, $char[$i-1], $maxchar[1], $modifier),
-                        _octets(2, $minchar[2], $maxchar[2], $modifier),
-                        _octets(3, $minchar[3], $char[$i+1], $modifier);
+                        _octets(1, $char[$i-1], &maxchar(1), $modifier),
+                        _octets(2, &minchar(2), &maxchar(2), $modifier),
+                        _octets(3, &minchar(3), $char[$i+1], $modifier);
                 }
                 elsif (length($char[$i+1]) == 4) {
                     push @charlist,
-                        _octets(1, $char[$i-1], $maxchar[1], $modifier),
-                        _octets(2, $minchar[2], $maxchar[2], $modifier),
-                        _octets(3, $minchar[3], $maxchar[3], $modifier),
-                        _octets(4, $minchar[4], $char[$i+1], $modifier);
+                        _octets(1, $char[$i-1], &maxchar(1), $modifier),
+                        _octets(2, &minchar(2), &maxchar(2), $modifier),
+                        _octets(3, &minchar(3), &maxchar(3), $modifier),
+                        _octets(4, &minchar(4), $char[$i+1], $modifier);
                 }
             }
             elsif (length($char[$i-1]) == 2) {
                 if (length($char[$i+1]) == 3) {
                     push @charlist,
-                        _octets(2, $char[$i-1], $maxchar[2], $modifier),
-                        _octets(3, $minchar[3], $char[$i+1], $modifier);
+                        _octets(2, $char[$i-1], &maxchar(2), $modifier),
+                        _octets(3, &minchar(3), $char[$i+1], $modifier);
                 }
                 elsif (length($char[$i+1]) == 4) {
                     push @charlist,
-                        _octets(2, $char[$i-1], $maxchar[2], $modifier),
-                        _octets(3, $minchar[3], $maxchar[3], $modifier),
-                        _octets(4, $minchar[4], $char[$i+1], $modifier);
+                        _octets(2, $char[$i-1], &maxchar(2), $modifier),
+                        _octets(3, &minchar(3), &maxchar(3), $modifier),
+                        _octets(4, &minchar(4), $char[$i+1], $modifier);
                 }
             }
             elsif (length($char[$i-1]) == 3) {
                 if (length($char[$i+1]) == 4) {
                     push @charlist,
-                        _octets(3, $char[$i-1], $maxchar[3], $modifier),
-                        _octets(4, $minchar[4], $char[$i+1], $modifier);
+                        _octets(3, $char[$i-1], &maxchar(3), $modifier),
+                        _octets(4, &minchar(4), $char[$i+1], $modifier);
                 }
             }
             else {
